
#include "Remembrance.h"

/** Allocates and returns a remembrance struct initialised to hold
    two elements in the vector, the first of which is set to zero.
    The vector is pre-allocated to be able to hold four elements.
    Caller should set the second element to the appropiate dimension. */
struct remembrance* remembrance_init() {
    struct remembrance *ret = malloc( sizeof( struct remembrance ) );
    if( ret == NULL )
         fprintf(stderr, "remembrance_init(): Error during initialisation!\n");
    else {
        ret->size = 2;
        ret->alloc = 4;
        ret->vector = malloc( ret->alloc*sizeof( long ) );
        ret->vector[0] = 0;
    }
    return ret;
}

/** Used for sorting longs from low to high. */
int remembrance_compare( const void *a, const void *b) {
    return *(long*)a - *(long*)b;
}

/** Will free a remembrance struct. */
int remembrance_destroy( struct remembrance * m ) {
    if( m == NULL ) {
        fprintf( stderr, "remembrance_destroy(): Null pointer supplied!\n" );
        return FALSE;
    }
    if( m->vector == NULL ) {
        fprintf( stderr, "remembrance_destroy(): Invalid remembrance structure!\n" );
        return FALSE;
    }
    free( m->vector );
    free( m );
    return TRUE;
}

long * remembrance_get( struct remembrance * const m ) {
    if( m == NULL ) {
        fprintf( stderr, "remembrance_get(): Null pointer supplied!\n" );
        return NULL;
    }
    if( m->vector == NULL ) {
        fprintf( stderr, "remembrance_get(): Invalid remembrance struct!\n" );
        return NULL;
    }
    qsort( (void*)m->vector, m->size, sizeof(long), remembrance_compare );
    return m->vector;
}

